@@ -1,5 +1,3 @@
-from __future__ import annotations
-
 import abc
 import logging
 import subprocess
@@ -7,7 +5,7 @@ import sys
 from os.path import exists as path_exists
 from os.path import expanduser
 from os.path import join as path_join
-from typing import Dict, List, Optional, Set, Type
+from typing import Dict, List, Optional, Set
 
 if sys.version_info[0] == 2:  # Python 2
 
@@ -26,14 +24,13 @@ if not _logger.handlers:
 
 
 class AddonsRegistry(object):
-    def __init__(self, *addon_types):
-        # type: (List[Type[AddonsSuffix]]) -> AddonsRegistry
-        base_addon_type = {GitSubDirOdooAddons, LocalSubDirOdooAddons, GitOdooAddons, LocalOdooAddons}
-        self.addon_types = base_addon_type | set(addon_types)
+    def __init__(self):
+        # type: () -> AddonsRegistry
+        self.addon_types = [LocalSubDirOdooAddons, GitSubDirOdooAddons, LocalOdooAddons, GitOdooAddons]
 
     def parse_env(self, env_vars=None):
         # type: (Dict[str, str]) -> Set[OdooAddonsDef]
-        founded = []
+        founded = {}
         if env_vars.get("ODOO_PATH"):
             if "ADDONS_LOCAL_SRC_ODOO_ADDONS" not in env_vars:
                 env_vars["ADDONS_LOCAL_SRC_ODOO_ADDONS"] = path_join(env_vars["ODOO_PATH"], "addons")
@@ -43,8 +40,9 @@ class AddonsRegistry(object):
         for k, v in env_vars.items():
             addons = self.parse_key(k)
             if addons:
-                founded.append(addons)
-        return {f.extract(env_vars) for f in founded}
+                if addons.identifier not in founded or addons.prefix != GitOdooAddons._prefix:
+                    founded[addons.identifier] = addons
+        return {f.extract(env_vars) for f in founded.values()}
 
     def parse_key(self, env_key):
         # type: (str) -> Optional[AddonsSuffix]
@@ -70,15 +68,15 @@ class OdooAddonsDef(ABC):
 
 class AddonsInstaller:
     @staticmethod
-    def exec_cmd(cmd, force_log=False):
+    def exec_cmd(cmd, force_log=True):
         if not cmd:
             return 0
         if force_log:
             _logger.info(" ".join(cmd))
-        return AddonsInstaller.rasie_if_error(subprocess.Popen(cmd).wait())
+        return AddonsInstaller.exit_if_error(subprocess.Popen(cmd).wait())
 
     @staticmethod
-    def rasie_if_error(error_no):
+    def exit_if_error(error_no):
         if error_no:
             sys.exit(error_no)
         return error_no
@@ -102,13 +100,14 @@ class AddonsInstaller:
             _logger.debug("No package.json founded in %s", path_npm)
 
     @staticmethod
-    def install(git_addons):
+    def install(addons):
         # type: (OdooAddonsDef) -> None
-        _logger.info("install %s", git_addons)
+        _logger.info("install %s", addons)
         try:
-            AddonsInstaller.exec_cmd(git_addons.install_cmd())
-            AddonsInstaller.install_py_requirements(git_addons.addons_path)
-            AddonsInstaller.install_npm_package(git_addons.addons_path)
+            for cmd in addons.install_cmd():
+                AddonsInstaller.exec_cmd(cmd, True)
+            AddonsInstaller.install_py_requirements(addons.addons_path)
+            AddonsInstaller.install_npm_package(addons.addons_path)
         except Exception as e:
             _logger.exception("Error", exc_info=e)
             sys.exit(1)
@@ -192,7 +191,7 @@ class AddonsSuffix(ABC):
 
 
 class GitOdooAddons(AddonsSuffix):
-
+    _prefix = "ADDONS_GIT"
     PROTOCLE_HTTPS = "https"
     PROTOCLE_SSH = "ssh"
     PROTOCLE_PUBLIC = "public"
@@ -203,10 +202,10 @@ class GitOdooAddons(AddonsSuffix):
     }
 
     def __init__(self, base_key):
-        super(GitOdooAddons, self).__init__("ADDONS_GIT", base_key)
+        super(GitOdooAddons, self).__init__(GitOdooAddons._prefix, base_key)
         self.BRANCH = self.create_key("BRANCH", default="master")
         self.CLONE_PATH = self.create_key("CLONE_PATH")
-        self.PULL_OPTIONS = self.create_key("PULL_OPTIONS", default="--depth=1 --quiet --single-branch")
+        self.PULL_OPTIONS = self.create_key("PULL_OPTIONS", default="--depth=1,--quiet,--single-branch")
         self.HTTPS_LOGIN = self.create_key("HTTPS_LOGIN", have_default=False)
         self.HTTPS_PASSWORD = self.create_key("HTTPS_PASSWORD", have_default=False)
         self.SSH_KEY = self.create_key("SSH_KEY", have_default=False)
@@ -249,7 +248,7 @@ class GitOdooAddons(AddonsSuffix):
                 return
             clone_cmd = ["git", "clone"]
             if self.pull_options:
-                clone_cmd.append(self.pull_options)
+                clone_cmd.extend(self.pull_options)
             if self.branch:
                 clone_cmd.append("-b")
                 clone_cmd.append(self.branch)
@@ -314,12 +313,20 @@ class GitOdooAddons(AddonsSuffix):
         if self.CLONE_PATH.default_key in env_vars and self.CLONE_PATH.full_key not in env_vars:
             # If default key in env_vars but not dedicated key then we add identifier in lower case to the clone path
             res[self.CLONE_PATH] = path_join(res[self.CLONE_PATH], self.identifier.lower())
+        pull_option = res[self.PULL_OPTIONS]
+
+        if isinstance(pull_option, str):
+            pull_option = pull_option.split(",")
+        if "..." in pull_option:
+            idx = pull_option.index("...")
+            pull_option.pop(idx)
+            pull_option[idx:idx] = self.PULL_OPTIONS.default_value.split(",")
         return GitOdooAddons.Result(
             name=self.identifier,
             git_path=res[self.NAME],
             branch=res[self.BRANCH],
             clone_path=res[self.CLONE_PATH],
-            pull_options=res[self.PULL_OPTIONS],
+            pull_options=pull_option,
             https_login=res.get(self.HTTPS_LOGIN),
             https_password=res.get(self.HTTPS_PASSWORD),
             ssh_key=res.get(self.SSH_KEY),
@@ -329,6 +336,8 @@ class GitOdooAddons(AddonsSuffix):
 
 
 class LocalOdooAddons(AddonsSuffix):
+    _prefix = "ADDONS_LOCAL"
+
     def __init__(self, base_key):
         super(LocalOdooAddons, self).__init__("ADDONS_LOCAL", base_key)
         self.BASE_PATH = self.create_key("BASE_PATH", default="/")
